#include "customer.hh"
#include "shop.hh"
#include "mall.hh"
#include "world.hh"
#include <iostream>

using namespace std;

namespace {
  void print_count(int count, const string& item) {
    if(count == 1)
      cout << "There is one " << item << " ";
    else
      cout << "There are " << count << " " << item << "s ";
  }
}

void Customer::look(bool verbose) {
  if(!shop) {
    cout << "You are in your car." << endl;
    if(!verbose)
      return;
    Malls malls = World::instance().malls();
    int count = malls.size();
    if(count == 0)
      cout << "There is nowhere to go." << endl;
    else {
      print_count(count, "mall");  cout << "you can go to:" << endl;
      for(Malls::const_iterator
	    i = malls.begin(); i != malls.end(); i++)
	cout << "  " << (*i)->name() << endl;
    }
  } else {
    cout << "You are in " << shop->name() << "." << std::endl;
    if(!verbose)
      return;
    {
      const Shops& shops = shop->shops();
      int count = shops.size();
      if((count == 0) && dynamic_cast<Mall*>(shop))
	cout << "There are no shops here." << endl;
      else {
	print_count(count, "shop");  cout << "here:" << endl;
	for(Shops::const_iterator
	      i = shops.begin(); i != shops.end(); i++)
	  cout << "  " << (*i)->name() << endl;
      }
    }
    {
      Products products = shop->products();
      int count = products.size();
      if(count == 0)
	cout << "There is nothing to buy here." << endl;
      else {
	print_count(count, "product");  cout << "here:" << endl;
	for(Products::const_iterator
	      i = products.begin(); i != products.end(); i++)
	  cout << "  " << (*i)->name << " (" << (*i)->price << ")" << endl;
      }
    }
    {
      Products products = cart.products();
      int count = products.size();
      if(count == 0)
	cout << "Your shopping cart is empty." << endl;
      else {
	print_count(count, "product");
	cout << "in your shopping cart:" << endl;
	Price total;
	for(Products::const_iterator
	      i = products.begin(); i != products.end(); i++) {
	  cout << "  " << (*i)->name << " (" << (*i)->price << ")" << endl;
	  total += (*i)->price;
	}
	cout << "  Total cost: " << total << endl;
      }
    }
  }
}

void Customer::go(Shop* to_shop) {
  if(shop)
    shop->exit(this);
  if(to_shop) 
    to_shop->enter(this);
  Mall* mall = dynamic_cast<Mall*>(to_shop);
  if(mall)
    cart = mall->get_cart();
  shop = to_shop;
}

void Customer::leave() {
  if(shop) {
    shop->exit(this);
    shop = shop->parent();
    if(shop)
      shop->enter(this);
  }
}
