#ifndef INTERPRETER_HH
#define INTERPRETER_HH
#include <deque>
#include <string>

// help|?
// load <filename>
// undo
// go [to] <mall|shop>
// take <product name>
// look
// leave
// build mall|<shop type> called <name>
// and
// then
// quit

class Customer;

typedef std::deque<std::string> Tokens;

class SyntaxTree;
typedef std::deque<SyntaxTree*> Trees;

class SyntaxTree {
public:
  virtual ~SyntaxTree() {}
  virtual void add(SyntaxTree *tree) { _children.push_back(tree); }
  virtual void interpret(Customer&) = 0;
protected:
  Trees _children;
};

class Sequence: public SyntaxTree {
public:
  void interpret(Customer&);
};

class CommandRep: public SyntaxTree {
public:
  CommandRep(const std::string& name) : _name(name) {}
  void interpret(Customer&);
private:
  std::string _name;
};


class Arguments: public SyntaxTree {
public:
  Arguments(const Tokens& tokens) : _tokens(tokens) {}
  void add(SyntaxTree *tree) {}  // ignore
  void interpret(Customer&);
  Tokens& tokens() { return _tokens; }

private:
  Tokens _tokens;
};

#endif
