#include "parser.hh"
#include "interpreter.hh"
#include <vector>

using std::string;

namespace {

/// Return true if c is a whitespace character.
bool is_ws(char c) {
  switch(c) {
  case ' ': case '\t': case '\n': case '\r': return true;
  default: return false;
  }
}

Tokens split_by_ws(const string& s) {
  Tokens tokens;
  int start = -1;
  int size = s.size();
  for(int i = 0; i < size; i++) {
    bool ws = is_ws(s[i]);
    if(start >= 0 && ws) {
      tokens.push_back(s.substr(start, i - start));
      start = -1;
    } else if(start < 0 && !ws) {
      start = i;
    }
  }
  if(start >= 0)
    tokens.push_back(s.substr(start));
  return tokens;
}

template<class Foo>
void append(Foo& to, Foo from) {
//   for(Foo::const_iterator
// 	i = from.begin(); i != from.end(); i++)
//     to.push_back(*i);
  while(!from.empty()) {
    to.push_back(from.front());
    from.pop_front();
  }
}

template<class Foo>
string join(Foo strings, const string& padding) {
  string tmp = "";
  while(!strings.empty()) {
    tmp += strings.front();  strings.pop_front();
    if(strings.empty())
      break;
    tmp += padding;
  }
  return tmp;
}

Tokens split(const string& s, const string& token) {
  Tokens tokens;
  unsigned int end = s.find(token);
  if(end > s.length())
    end = s.length();
  tokens.push_back(s.substr(0, end));
  end += token.length();
  if(end < s.length())
    append(tokens, split(s.substr(end), token));
  return tokens;
}

SyntaxTree* parse_expression(string code) {
  Tokens tokens = split_by_ws(code);
  if(tokens.empty())
    return 0;
  CommandRep *command = new CommandRep(tokens.front());
  tokens.pop_front();
  code = " " + join(tokens, "  ") + " ";
  Tokens arguments = split(code, " and ");
  for(Tokens::const_iterator
	i = arguments.begin(); i != arguments.end(); i++)
    command->add(new Arguments(split_by_ws(*i)));
  return command;
}

Sequence* build_sequence(const Trees& trees) {
  Sequence *sequence = new Sequence();
  for(Trees::const_iterator tree = trees.begin(); tree != trees.end(); tree++)
    sequence->add(*tree);
  return sequence;
}

} // namespace

SyntaxTree* parse(string code) {
  Tokens tokens = split_by_ws(code);
  if(tokens.empty())
    return 0;
  code = " " + join(tokens, "  ") + " ";
  Tokens expressions = split(code, " then ");
  Trees trees;
  for(Tokens::const_iterator
	i = expressions.begin(); i != expressions.end(); i++) {
    SyntaxTree *tree = parse_expression(*i);
    if(tree)
      trees.push_back(tree);
    //    std::cout << *i << std::endl;
  }
  if(trees.size() == 1) {
    return trees.front();
  } else {
    return build_sequence(trees);
  }
}

#ifdef PARSER_STAND_ALONE
int main() {
  //alarm(5);
  string line;
  std::cerr << "> " << std::flush;
  while(std::getline(std::cin, line)) {
    try {
      SyntaxTree *tree = parse(line);
      if(tree)
	tree->interpret();
    }
    catch(const ParseError &e) {
      std::cerr << e.what() << std::endl;
    }
    std::cerr << "> " << std::flush;
  }
}
#endif
