#ifndef SHOP_HH
#define SHOP_HH

#include <set>
#include <string>
#include "product.hh"

class Customer;
class Cart;

typedef std::set<Shop*> Shops;

class Shop {
public:
  typedef std::set<Customer*> Customers;
  
  Shop(const std::string& name, Shop* parent = 0)
    : _name(name), _parent(parent) {}
  virtual Shop* clone(const std::string& name, Shop* parent = 0) const = 0;
  virtual ~Shop() {}

  void enter(Customer *customer);
  void exit(Customer *customer);
  Shop* parent() const { return _parent; }
  const std::string& name() const { return _name; }
  const Customers& customers() const { return _customers; }
  const Products& products() const { return _products; }
  const Shops& shops() const { return _shops; }
  virtual void put_in_cart(Cart &cart, const Product *product) = 0;
  virtual void remove_from_cart(Cart &cart, const Product *product) = 0;
  virtual void add_shop(Shop *shop) { _shops.insert(shop); }

protected:
  std::string _name;
  Shop *_parent;
  ID _id;
  Shops _shops;
  Customers _customers;
  Products _products;
};

#endif
